@@ -27,10 +27,11 @@
 #include "config.h"
 #endif
 
+#include <xorg-server.h>
 #include "xf86Module.h"
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 3
 
 #include <X11/Xatom.h>
+#include <xf86.h>
 #include <xf86Xinput.h>
 #include <exevents.h>
 
@@ -38,6 +39,11 @@
 #include "synapticsstr.h"
 #include "synaptics-properties.h"
 
+#ifndef XATOM_FLOAT
+#define XATOM_FLOAT "FLOAT"
+#endif
+static Atom float_type;
+
 Atom prop_edges                 = 0;
 Atom prop_finger                = 0;
 Atom prop_tap_time              = 0;
@@ -46,6 +52,7 @@ Atom prop_tap_durations         = 0;
 Atom prop_tap_fast              = 0;
 Atom prop_middle_timeout        = 0;
 Atom prop_twofinger_pressure    = 0;
+Atom prop_twofinger_width       = 0;
 Atom prop_scrolldist            = 0;
 Atom prop_scrolledge            = 0;
 Atom prop_scrolltwofinger       = 0;
@@ -57,7 +64,6 @@ Atom prop_buttonscroll          = 0;
 Atom prop_buttonscroll_repeat   = 0;
 Atom prop_buttonscroll_time     = 0;
 Atom prop_off                   = 0;
-Atom prop_guestmouse            = 0;
 Atom prop_lockdrags             = 0;
 Atom prop_lockdrags_time        = 0;
 Atom prop_tapaction             = 0;
@@ -70,7 +76,12 @@ Atom prop_palm                  = 0;
 Atom prop_palm_dim              = 0;
 Atom prop_coastspeed            = 0;
 Atom prop_pressuremotion        = 0;
+Atom prop_pressuremotion_factor = 0;
 Atom prop_grab                  = 0;
+Atom prop_gestures              = 0;
+Atom prop_capabilities          = 0;
+Atom prop_resolution            = 0;
+Atom prop_area                  = 0;
 
 static Atom
 InitAtom(DeviceIntPtr dev, char *name, int format, int nvalues, int *values)
@@ -108,13 +119,37 @@ InitAtom(DeviceIntPtr dev, char *name, int format, int nvalues, int *values)
     return atom;
 }
 
+static Atom
+InitFloatAtom(DeviceIntPtr dev, char *name, int nvalues, float *values)
+{
+    Atom atom;
+
+    atom = MakeAtom(name, strlen(name), TRUE);
+    XIChangeDeviceProperty(dev, atom, float_type, 32, PropModeReplace,
+                           nvalues, values, FALSE);
+    XISetDevicePropertyDeletable(dev, atom, FALSE);
+    return atom;
+}
+
 void
 InitDeviceProperties(LocalDevicePtr local)
 {
     SynapticsPrivate *priv = (SynapticsPrivate *) local->private;
-    SynapticsSHM *para = priv->synpara;
-
+    SynapticsParameters *para = &priv->synpara;
     int values[9]; /* we never have more than 9 values in an atom */
+    float fvalues[4]; /* never have more than 4 float values */
+
+    float_type = XIGetKnownProperty(XATOM_FLOAT);
+    if (!float_type)
+    {
+        float_type = MakeAtom(XATOM_FLOAT, strlen(XATOM_FLOAT), TRUE);
+        if (!float_type)
+        {
+            xf86Msg(X_ERROR, "%s: Failed to init float atom. "
+                             "Disabling property support.\n", local->name);
+            return;
+        }
+    }
 
     values[0] = para->left_edge;
     values[1] = para->right_edge;
@@ -141,6 +176,8 @@ InitDeviceProperties(LocalDevicePtr local)
                                    32, 1, &para->emulate_mid_button_time);
     prop_twofinger_pressure = InitAtom(local->dev, SYNAPTICS_PROP_TWOFINGER_PRESSURE,
                                        32, 1, &para->emulate_twofinger_z);
+    prop_twofinger_width = InitAtom(local->dev, SYNAPTICS_PROP_TWOFINGER_WIDTH,
+                                       32, 1, &para->emulate_twofinger_w);
 
     values[0] = para->scroll_dist_vert;
     values[1] = para->scroll_dist_horiz;
@@ -154,7 +191,11 @@ InitDeviceProperties(LocalDevicePtr local)
     values[1] = para->scroll_twofinger_horiz;
     prop_scrolltwofinger = InitAtom(local->dev, SYNAPTICS_PROP_SCROLL_TWOFINGER,8, 2, values);
 
-    /* FIXME: MISSING: speed is a float */
+    fvalues[0] = para->min_speed;
+    fvalues[1] = para->max_speed;
+    fvalues[2] = para->accl;
+    fvalues[3] = para->trackstick_speed;
+    prop_speed = InitFloatAtom(local->dev, SYNAPTICS_PROP_SPEED, 4, fvalues);
 
     values[0] = para->edge_motion_min_z;
     values[1] = para->edge_motion_max_z;
@@ -165,16 +206,19 @@ InitDeviceProperties(LocalDevicePtr local)
     prop_edgemotion_speed = InitAtom(local->dev, SYNAPTICS_PROP_EDGEMOTION_SPEED, 32, 2, values);
     prop_edgemotion_always = InitAtom(local->dev, SYNAPTICS_PROP_EDGEMOTION, 8, 1, &para->edge_motion_use_always);
 
-    values[0] = para->updown_button_scrolling;
-    values[1] = para->leftright_button_scrolling;
-    prop_buttonscroll = InitAtom(local->dev, SYNAPTICS_PROP_BUTTONSCROLLING, 8, 2, values);
+    if (priv->has_scrollbuttons)
+    {
+        values[0] = para->updown_button_scrolling;
+        values[1] = para->leftright_button_scrolling;
+        prop_buttonscroll = InitAtom(local->dev, SYNAPTICS_PROP_BUTTONSCROLLING, 8, 2, values);
+
+        values[0] = para->updown_button_repeat;
+        values[1] = para->leftright_button_repeat;
+        prop_buttonscroll_repeat = InitAtom(local->dev, SYNAPTICS_PROP_BUTTONSCROLLING_REPEAT, 8, 2, values);
+        prop_buttonscroll_time = InitAtom(local->dev, SYNAPTICS_PROP_BUTTONSCROLLING_TIME, 32, 1, &para->scroll_button_repeat);
+    }
 
-    values[0] = para->updown_button_repeat;
-    values[1] = para->leftright_button_repeat;
-    prop_buttonscroll_repeat = InitAtom(local->dev, SYNAPTICS_PROP_BUTTONSCROLLING_REPEAT, 8, 2, values);
-    prop_buttonscroll_time = InitAtom(local->dev, SYNAPTICS_PROP_BUTTONSCROLLING_TIME, 32, 1, &para->scroll_button_repeat);
     prop_off = InitAtom(local->dev, SYNAPTICS_PROP_OFF, 8, 1, &para->touchpad_off);
-    prop_guestmouse = InitAtom(local->dev, SYNAPTICS_PROP_GUESTMOUSE, 8, 1, &para->guestmouse_off);
     prop_lockdrags = InitAtom(local->dev, SYNAPTICS_PROP_LOCKED_DRAGS, 8, 1, &para->locked_drags);
     prop_lockdrags_time = InitAtom(local->dev, SYNAPTICS_PROP_LOCKED_DRAGS_TIMEOUT, 32, 1, &para->locked_drag_time);
 
@@ -185,7 +229,10 @@ InitDeviceProperties(LocalDevicePtr local)
     prop_clickaction = InitAtom(local->dev, SYNAPTICS_PROP_CLICK_ACTION, 8, MAX_CLICK, values);
 
     prop_circscroll = InitAtom(local->dev, SYNAPTICS_PROP_CIRCULAR_SCROLLING, 8, 1, &para->circular_scrolling);
-    /* FIXME: missing: scroll_dist_circ is a float */
+
+    fvalues[0] = para->scroll_dist_circ;
+    prop_circscroll_dist = InitFloatAtom(local->dev, SYNAPTICS_PROP_CIRCULAR_SCROLLING_DIST, 1, fvalues);
+
     prop_circscroll_trigger = InitAtom(local->dev, SYNAPTICS_PROP_CIRCULAR_SCROLLING_TRIGGER, 8, 1, &para->circular_trigger);
     prop_circpad = InitAtom(local->dev, SYNAPTICS_PROP_CIRCULAR_PAD, 8, 1, &para->circular_pad);
     prop_palm = InitAtom(local->dev, SYNAPTICS_PROP_PALM_DETECT, 8, 1, &para->palm_detect);
@@ -195,16 +242,42 @@ InitDeviceProperties(LocalDevicePtr local)
 
     prop_palm_dim = InitAtom(local->dev, SYNAPTICS_PROP_PALM_DIMENSIONS, 32, 2, values);
 
-    /* FIXME: missing, coastspeed is a float */
+    fvalues[0] = para->coasting_speed;
+    fvalues[1] = para->coasting_friction;
+    prop_coastspeed = InitFloatAtom(local->dev, SYNAPTICS_PROP_COASTING_SPEED, 2, fvalues);
 
     values[0] = para->press_motion_min_z;
     values[1] = para->press_motion_max_z;
     prop_pressuremotion = InitAtom(local->dev, SYNAPTICS_PROP_PRESSURE_MOTION, 32, 2, values);
 
-    /* FIXME: missing, motion_min/max is a float */
+    fvalues[0] = para->press_motion_min_factor;
+    fvalues[1] = para->press_motion_max_factor;
+
+    prop_pressuremotion_factor = InitFloatAtom(local->dev, SYNAPTICS_PROP_PRESSURE_MOTION_FACTOR, 2, fvalues);
 
     prop_grab = InitAtom(local->dev, SYNAPTICS_PROP_GRAB, 8, 1, &para->grab_event_device);
 
+    values[0] = para->tap_and_drag_gesture;
+    prop_gestures = InitAtom(local->dev, SYNAPTICS_PROP_GESTURES, 8, 1, values);
+
+    values[0] = priv->has_left;
+    values[1] = priv->has_middle;
+    values[2] = priv->has_right;
+    values[3] = priv->has_double;
+    values[4] = priv->has_triple;
+    values[5] = priv->has_pressure;
+    values[6] = priv->has_width;
+    prop_capabilities = InitAtom(local->dev, SYNAPTICS_PROP_CAPABILITIES, 8, 7, values);
+
+    values[0] = para->resolution_vert;
+    values[1] = para->resolution_horiz;
+    prop_resolution = InitAtom(local->dev, SYNAPTICS_PROP_RESOLUTION, 32, 2, values);
+
+    values[0] = para->area_left_edge;
+    values[1] = para->area_right_edge;
+    values[2] = para->area_top_edge;
+    values[3] = para->area_bottom_edge;
+    prop_area = InitAtom(local->dev, SYNAPTICS_PROP_AREA, 32, 4, values);
 }
 
 int
@@ -213,8 +286,8 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
 {
     LocalDevicePtr local = (LocalDevicePtr) dev->public.devicePrivate;
     SynapticsPrivate *priv = (SynapticsPrivate *) local->private;
-    SynapticsSHM *para = priv->synpara;
-    SynapticsSHM tmp;
+    SynapticsParameters *para = &priv->synpara;
+    SynapticsParameters tmp;
 
     /* If checkonly is set, no parameters may be changed. So just let the code
      * change temporary variables and forget about it. */
@@ -298,6 +371,12 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
             return BadMatch;
 
         para->emulate_twofinger_z = *(INT32*)prop->data;
+    } else if (property == prop_twofinger_width)
+    {
+        if (prop->size != 1 || prop->format != 32 || prop->type != XA_INTEGER)
+            return BadMatch;
+
+        para->emulate_twofinger_w = *(INT32*)prop->data;
     } else if (property == prop_scrolldist)
     {
         INT32 *dist;
@@ -329,7 +408,17 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
         para->scroll_twofinger_horiz = twofinger[1];
     } else if (property == prop_speed)
     {
-        /* XXX */
+        float *speed;
+
+        if (prop->size != 4 || prop->format != 32 || prop->type != float_type)
+            return BadMatch;
+
+        speed = (float*)prop->data;
+        para->min_speed = speed[0];
+        para->max_speed = speed[1];
+        para->accl = speed[2];
+        para->trackstick_speed = speed[3];
+
     } else if (property == prop_edgemotion_pressure)
     {
         CARD32 *pressure;
@@ -369,6 +458,9 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
     {
         BOOL *scroll;
 
+        if (!priv->has_scrollbuttons)
+            return BadMatch;
+
         if (prop->size != 2 || prop->format != 8 || prop->type != XA_INTEGER)
             return BadMatch;
 
@@ -380,6 +472,9 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
     {
         BOOL *repeat;
 
+        if (!priv->has_scrollbuttons)
+            return BadMatch;
+
         if (prop->size != 2 || prop->format != 8 || prop->type != XA_INTEGER)
             return BadMatch;
 
@@ -388,6 +483,9 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
         para->leftright_button_repeat = repeat[1];
     } else if (property == prop_buttonscroll_time)
     {
+        if (!priv->has_scrollbuttons)
+            return BadMatch;
+
         if (prop->size != 1 || prop->format != 32 || prop->type != XA_INTEGER)
             return BadMatch;
 
@@ -405,12 +503,15 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
             return BadValue;
 
         para->touchpad_off = off;
-    } else if (property == prop_guestmouse)
+    } else if (property == prop_gestures)
     {
+        BOOL *gestures;
+
         if (prop->size != 1 || prop->format != 8 || prop->type != XA_INTEGER)
             return BadMatch;
 
-        para->guestmouse_off = *(BOOL*)prop->data;
+        gestures = (BOOL*)prop->data;
+        para->tap_and_drag_gesture = gestures[0];
     } else if (property == prop_lockdrags)
     {
         if (prop->size != 1 || prop->format != 8 || prop->type != XA_INTEGER)
@@ -456,7 +557,13 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
 
     } else if (property == prop_circscroll_dist)
     {
-        /* FIXME */
+        float circdist;
+
+        if (prop->size != 1 || prop->format != 32 || prop->type != float_type)
+            return BadMatch;
+
+        circdist = *(float*)prop->data;
+        para->scroll_dist_circ = circdist;
     } else if (property == prop_circscroll_trigger)
     {
         int trigger;
@@ -494,14 +601,21 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
         para->palm_min_z     = dim[1];
     } else if (property == prop_coastspeed)
     {
-        /* FIXME */
-    } else if (property == prop_pressuremotion)
-    {
-        INT32 *press;
-        if (prop->size != 2 || prop->format != 32 || prop->type != XA_INTEGER)
+        float *coast_speeds;
+
+        if (prop->size != 2 || prop->format != 32 || prop->type != float_type)
             return BadMatch;
 
-        press = (INT32*)prop->data;
+        coast_speeds = (float*)prop->data;
+        para->coasting_speed = coast_speeds[0];
+        para->coasting_friction = coast_speeds[1];
+    } else if (property == prop_pressuremotion)
+    {
+        float *press;
+        if (prop->size != 2 || prop->format != 32 || prop->type != float_type)
+            return BadMatch;
+
+        press = (float*)prop->data;
         if (press[0] > press[1])
             return BadValue;
 
@@ -513,10 +627,30 @@ SetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
             return BadMatch;
 
         para->grab_event_device = *(BOOL*)prop->data;
+    } else if (property == prop_capabilities)
+    {
+        /* read-only */
+        return BadValue;
+    } else if (property == prop_resolution)
+    {
+        /* read-only */
+        return BadValue;
+    } else if (property == prop_area)
+    {
+        INT32 *area;
+        if (prop->size != 4 || prop->format != 32 || prop->type != XA_INTEGER)
+            return BadMatch;
+
+        area = (INT32*)prop->data;
+        if ((((area[0] != 0) && (area[1] != 0)) && (area[0] > area[1]) ) || (((area[2] != 0) && (area[3] != 0)) && (area[2] > area[3])))
+            return BadValue;
+
+        para->area_left_edge   = area[0];
+        para->area_right_edge  = area[1];
+        para->area_top_edge    = area[2];
+        para->area_bottom_edge = area[3];
     }
 
     return Success;
 }
 
-#endif
-
